# -*- coding: utf-8 -*-

"""
upraise.decorators
~~~~~~~~~~~~~~~~~~

Various decorators for templating, RESTful dispatching and data export.

:date: $Date: 2007-07-20 22:20:04 +0200 (Fr, 20 Jul 2007) $
:revision: $Rev: 161 $
:copyright: 2006-2007 Jochen Kupperschmidt
:license: MIT, see LICENSE for details.
"""

from paste.wsgiwrappers import WSGIResponse

from upraise.util import config


__all__ = ['use_template', 'restrict_method_to', 'deflect_on_method',
    'jsonify', 'yamlify']


# ---------------------------------------------------------------- #
# templating

def use_template(arg):
    """Decorate a callable to wrap its return value in a template and that in
    a response object.

    This decorator expects the decorated callable to return a dictionary of
    objects that should be added to the template context (or ``None``).

    The name of the template to render can be either specified as argument or,
    if not present, will be determined by concatenating the callable's class
    and method name (format: 'class_method'). This, of course, will only work
    if the callable is a method, not a function.

    The rendered template string will be wrapped in a ``WSGIResponse`` object
    and returned.

    To keep this decorator independent of templating systems, you will have to
    specify your own callable that renders templates. Its arguments are the
    name of the template to render and a dictionary of variables to add to the
    template's context. It is expected to return the result as a string.

    Example implementation::

        def render_template(name, vars):
            '''Render the named template using the given variables.'''
            tpl = some_template_package.load_template(name + '.tpl')
            tpl.update_context(vars)
            return tpl.serialize('some_encoding')
    """
    if hasattr(arg, '__call__'):
        return _create_response(arg)
    def wrapper(func):
        return _create_response(func, arg)
    return wrapper

def _create_response(func, name=None):
    """Render a template and wrap it into a response object."""
    if name is None:
        name = '%s_%s' % (func.__module__.split('.')[-1], func.__name__)
    def inner(*args, **kwargs):
        vars = func(*args, **kwargs)

        # If it is a response object, skip templating and pass it through.
        if isinstance(vars, WSGIResponse):
            return vars

        # Make sure we have a dictionary of objects to be added to the context.
        if vars is None:
            vars = {}
        elif not isinstance(vars, dict):
            raise TypeError(
                "Controller callable must return a 'dict' object or None.")

        # Render the template and wrap in a response object.
        if not hasattr(config, 'render_template_func'):
            raise NotImplementedError(
                "No callable 'render_template_func' for rendering templates "
                "was set.")
        return WSGIResponse(config.render_template_func(name, vars))
    return inner


# ---------------------------------------------------------------- #
# RESTful request handling
# (REST = Representational State Transfer)

def restrict_method_to(*methods):
    """Restricts the allowed HTTP methods to those given as arguments."""
    def wrapper(func, *args, **kwargs):
        if request.method not in methods:
            resp = WSGIResponse(code=405)
            resp.headers['Allow'] = ','.join(methods)
            return resp
        return func(*args, **kwargs)
    return wrapper

def deflect_on_method(**method_map):
    """Pass the call to another controller method."""
    def wrapper(func, self, *args, **kwargs):
        try:
            action = getattr(self, method_map[request.method])
            return action(self, *args, **kwargs)
        except KeyError:
            return func(self, *args, **kwargs)
    return wrapper


# ---------------------------------------------------------------- #
# data export/serialization

def jsonify(func):
    """Send the data returned by the decorated function as JSON."""
    import simplejson
    def wrapper(*args, **kwargs):
        data = simplejson.dumps(func(*args, **kwargs))
        return WSGIResponse(data, 'text/javascript')
    return wrapper

def yamlify(func):
    """Send the data returned by the decorated function as YAML."""
    import yaml
    def wrapper(*args, **kwargs):
        data = yaml.safe_dump(func(*args, **kwargs))
        return WSGIResponse(data, 'text/plain')
    return wrapper
